from django.contrib import admin
from models import *
from django.utils.translation import ugettext_lazy as _
import reversion
from crm.admin import CustomerAdmin
from django.core.urlresolvers import reverse
from mailchimp import utils
from crm.forms import AddToMailingListForm
from django.http import HttpResponseRedirect
from django.template import RequestContext
from django.shortcuts import render_to_response
from mailchimp.chimpy.chimpy import ChimpyException


class CustomerAdminMixin(admin.ModelAdmin):
    search_fields = ['customer__initials', 'customer__first_name', 'customer__last_name',
                     'customer__last_name_prefix', 'customer__phone_number', 'customer__phone_number2',
                     'customer__city', 'customer__nationality',
                     'customer__email', 'customer__address', 'customer__postal_code',
                     'customer__identification_number',
                     'customer__comments', 'customer__source']

    def queryset(self, request):
        return super(CustomerAdminMixin, self).queryset(request).select_related('customer')

    def get_email(self, obj):
        return obj.customer.email

    get_email.allow_tags = True
    get_email.short_description = _('Email')

    def get_phone_numbers(self, obj):
        return obj.customer.phone_number+' '+obj.customer.phone_number2

    get_phone_numbers.allow_tags = True
    get_phone_numbers.short_description = _('Phone number')

    def get_alternative(self, obj):
        relations = obj.customer.relationships.all()
        if len(relations) > 0:
            return '<a href="%s">%s</a>' % (
                reverse('admin:crm_customer_change', args=(relations[0].pk,)), relations[0]
            )
        return None
    get_alternative.allow_tags = True
    get_alternative.short_description = _('relationship')

    def add_to_mailinglist(self, request, queryset):
        form = None

        if 'apply' in request.POST:
            form = AddToMailingListForm(request.POST)

            if form.is_valid():
                list_id = form.cleaned_data['list']
                connection = utils.get_connection()
                list = connection.get_list_by_id(list_id)
                count = 0
                false_count = 0
                for lead in queryset:
                    try:
                        list.subscribe(lead.customer.email, {'EMAIL': lead.customer.email,
                                                             'FNAME': lead.customer.first_name,
                                                             'LNAME': lead.customer.last_name_prefix+' '+lead.customer.last_name},
                                       double_optin=False,
                                       update_existing=True)
                        count += 1
                    except ChimpyException:
                        false_count += 1

                plural = ''
                if count != 1:
                    plural = 's'

                self.message_user(request,
                                  "Successfully added %d/%d customer%s to mailchimp list %s." % (count,
                                                                                                 count+false_count,
                                                                                                 plural,
                                                                                                 list.name))
                return HttpResponseRedirect(request.get_full_path())

        if not form:
            form = AddToMailingListForm(initial={'_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME)})

        return render_to_response('admin/add_to_list.html', {'customers': queryset,
                                                             'form': form}, context_instance=RequestContext(request))

    actions = ['add_to_mailinglist']


class BuyerAdmin(reversion.VersionAdmin, CustomerAdminMixin):
    list_display = ['customer', 'house', 'get_email', 'get_phone_numbers', 'option', 'option_from', 'option_until',
                    'signed_agreement', 'signed_agreement_date', 'notary']
    list_filter = ['option', 'house__parent', 'signed_agreement', 'notary']

    def queryset(self, request):
        return super(BuyerAdmin, self).queryset(request).select_related('customer')

    fieldsets = (
        (None, {
            'fields': ('customer', 'house', ('option', 'option_from', 'option_until'), ('signed_agreement',
                                                                                        'signed_agreement_date'),
                       'notary', 'finance_status')
        }),
    )


class BuyerInline(admin.StackedInline):
    model = Buyer
    fk_name = 'customer'
    verbose_name = _('buyer information')
    verbose_name_plural = _('buyer information')
    fieldsets = (
        (None, {
            'classes': 'collapse',
            'fields': ('house', ('option', 'option_from', 'option_until'), ('signed_agreement',
                                                                            'signed_agreement_date'), 'notary',
                       'finance_status')
        }),
    )


class LeadInline(admin.StackedInline):
    model = Lead
    fk_name = 'customer'
    filter_vertical = ['houses', 'groups', 'house_kinds']
    verbose_name = _('lead')
    verbose_name_plural = _('lead')


class RealestateCustomerAdmin(CustomerAdmin):

    inlines = CustomerAdmin.inlines + [LeadInline, BuyerInline]


class LeadAdmin(reversion.VersionAdmin, CustomerAdminMixin):
    list_display = ['__unicode__', 'comment', 'get_email', 'get_phone_numbers', 'get_alternative', 'edited']
    list_filter = ['houses', 'house_kinds', 'groups', 'subgroups']
    filter_vertical = ['houses', 'groups', 'subgroups', 'house_kinds']


admin.site.unregister(Customer)
admin.site.register(Customer, RealestateCustomerAdmin)
admin.site.register(Buyer, BuyerAdmin)
admin.site.register(Lead, LeadAdmin)
